import os
import time
import urlparse
from behave import *
from selenium_basic_helpers import *
from selenium.common.exceptions import *
from selenium.webdriver.common.by import By
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

CHECK_INTERVAL_SECONDS = 0.01


#############################
#### BASIC HELPER METHODS####
#############################



#################################################
#### SUBSTEPS (FOR BASIC ACTIONS)################
#################################################
# Valid identifiers: 'css_selector, id', 'name' and 'class_name'


#****************
# WHEN STEPS
#****************

#Valid formats for screenshot: base64, file
@when('take screenshot as {output_format} and show description {text}')
def step(context, output_format, text):
    if not os.path.exists("snapshots"):
        os.makedirs("snapshots")
    if output_format == "base64":
        captured_img = context.browser.get_screenshot_as_base64()
        if os.path.isfile("snapshots/snapshots_report.html"):
            with open("snapshots/snapshots_report.html", "a") as snapshot_file:
                snapshot_file.write("""<br>"""+ text +"""<br><img src="data:image/png;base64,"""+captured_img+"""" />""")
        else:
            with open("snapshots/snapshots_report.html", "w") as snapshot_file:
                snapshot_file.write("""<br>"""+ text +"""<br><img src="data:image/png;base64,"""+captured_img+"""" />""")

    if output_format == "file":
        captured_img = context.browser.get_screenshot_as_file("snapshots/" + str(time.time()) + text + ".png")
        if captured_img == False:
            raise Exception("ERROR: cannot take snapshot...")

@when('opening the url {url}')
def step(context, url):
    context.browser.get(urlparse.urljoin(context.config.userdata.get('target_env'), url))

@when('I refresh the page')
def step(context):
    context.browser.refresh()

@when('I scroll down element {element_selector} {times} times')
def step(context, element_selector, times):
    element = find_element(context, element_selector, 'css_selector')
    try:
        action = ActionChains(context.browser)
        action.move_to_element(element).perform()
        element.click()
        for _ in range(times):
            element.send_keys(Keys.SPACE)
            time.sleep(0.1)
    except Exception as e:
        raise Exception('Error scrolling down web element')

@when('hover on element {element} identified by {identifier}')
def step(context, element, identifier):
    element = find_element(context, element, identifier)
    hov = ActionChains(context.browser).move_to_element(element)
    hov.perform()

@when('back to the previous page')
def step(context):
    context.browser.back()

@when('fill input field {field_name}, identified by {identifier} with the value {value}')
def steps(context, field_name, identifier, value, timeout = 5.0):
    input_field = find_element(context, field_name, identifier)
    input_field.clear()
    input_field.send_keys(value)
    while timeout > 0:
        if input_field.get_attribute('value') == value:
            return
        time.sleep(CHECK_INTERVAL_SECONDS)
        timeout -= CHECK_INTERVAL_SECONDS

@when('fill select field {field_name}, identified by {identifier} with the value {value}')
def steps(context, field_name, identifier, value):
    select_field = Select(find_element(context, field_name, identifier))
    select_field.select_by_visible_text(value)

@when('click on button {button_name} identified by {identifier}')
def step(context, button_name, identifier):
    button = find_element(context, button_name, identifier)
    button.click()

@when('click in {section} identified by {identifier}')
def step(context, section, identifier):
    element = find_element(context, section, identifier)
    scroll_element_into_view(context.browser, element)
    element.click()

@when('move to element {element}, identified by {identifier}')
def step(context, element, identifier):
    element = find_element(context, element, identifier)
    actions = ActionChains(context.browser)
    actions.move_to_element(element)
    actions.perform()

@when('select option {option} in combobox {element}, identified by {identifier}')
def step(context, option, element, identifier):
    element = find_element(context, element, identifier)
    select = Select(element)
    select.select_by_visible_text(option)

# Shortcuts:
@when('hovering {selector}')
def step(context, selector):
    element = find_element(context, selector, 'css_selector')
    hover = ActionChains(context.browser).move_to_element(element)
    hover.perform()

@when('clicking {selector}')
def step(context, selector):
    element = find_element(context, selector, 'css_selector')
    element.click()

@when('click in link with text {text}')
def step(context, text):
    wait = WebDriverWait(context.browser, 10)
    if context.config.userdata.get('browser_name') == "ie":
        link = find_element(context, ".text > div:nth-child(1) > a:nth-child(2)", 'css_selector')
    else:
        link = wait.until(EC.presence_of_element_located((By.LINK_TEXT, text)))
    if context.config.userdata.get('browser_name') == "chrome" or context.config.userdata.get('browser_name') == "ie":
        context.browser.execute_script("arguments[0].click();", link)
    else:
        link.click()

@when('uploading {image} to {selector}')
def step(context, image, selector):
    element = find_element(context, selector, 'css_selector')
    image_path = os.path.join(os.getcwd(), 'images', image)
    if context.config.userdata.get('browser_name') == "ie":
        fix_ie_script = "function fixie(){document.querySelector(\"" + selector  +  "\").style.opacity=1;}" + "fixie();"
        context.browser.execute_script(fix_ie_script)
    element.send_keys(image_path)

@when('upload the video {video} to {selector}')
def step(context, video, selector):
    element = find_element(context, selector, 'css_selector')
    video_path = os.path.join(os.getcwd(), 'videos', video)
    if context.config.userdata.get('browser_name') == "ie":
        fix_ie_script = "function fixie(){document.querySelector(\"" + selector  +  "\").style.opacity=1;}" + "fixie();"
        context.browser.execute_script(fix_ie_script)
    element.send_keys(video_path)

@when('upload the zip {zip_file} to {selector}')
def step(context, zip_file, selector):
    fix_ie_script = \
        "function fixie(){ " \
            "var el = document.querySelector(\"" + selector + "\");" \
            "el.style='';" \
        "}" + "fixie();"
    context.browser.execute_script(fix_ie_script)
    wait_for_element_to_appear(context, selector, float(5))
    element = find_element(context, selector, 'css_selector')
    zip_path = os.path.join(os.getcwd(), 'assets/zip', zip_file)
    element.send_keys(zip_path)

@when('upload the marker {img_file} to {selector}')
def step(context, img_file, selector):
    fix_ie_script = \
        "function fixie(){ " \
            "var el = document.querySelector(\"" + selector + "\");" \
            "el.style='';" \
        "}" + "fixie();"
    context.browser.execute_script(fix_ie_script)
    wait_for_element_to_appear(context, selector, float(5))
    element = find_element(context, selector, 'css_selector')
    zip_path = os.path.join(os.getcwd(), 'assets/img', img_file)
    element.send_keys(zip_path)

#options: up, down, left, right, enter
@when('send key {key} to {selector}')
def step(context, key, selector):
    element = find_element(context, selector, 'css_selector')
    if key == "up":
        element.send_keys(Keys.ARROW_UP)
    if key == "down":
        element.send_keys(Keys.ARROW_DOWN)
    if key == "left":
        element.send_keys(Keys.ARROW_LEFT)
    if key == "right":
        element.send_keys(Keys.ARROW_RIGHT)
    if key == "enter":
        element.send_keys(Keys.ENTER)
    else:
        element.send_keys(key)

@when('typing {value} in {selector}')
def steps(context, selector, value):
    element = find_element(context, selector, 'css_selector')
    if not element.is_displayed():
        assert False, "cannot type {value} in {selector}, element not displayed".format(**locals())
    element.clear()
    element.send_keys(value)

@when('selecting {option} in {selector}')
def step(context, option, selector):
    select = Select(find_element(context, selector, 'css_selector'))
    select.select_by_visible_text(option)

@when('drag element {source_element} into {target_element}')
def step(context, source_element, target_element):
    source = find_element(context, source_element, 'css_selector')
    target = find_element(context, target_element, 'css_selector')
    actionchains = ActionChains(context.browser)
    ele = actionchains.drag_and_drop(source, target)
    ele.perform()

@when('drag element {source_element} to positions x {x} and y {y}')
def step(context, source_element, x, y):
    source = find_element(context, source_element, 'css_selector')
    actionchains = ActionChains(context.browser)
    ele = actionchains.drag_and_drop_by_offset(source, int(x), int(y))
    ele.perform()

@when('make element {element_css_selector} visible')
def step(context, element_css_selector):
    element = find_element(context, element_css_selector, 'css_selector')
    js = "arguments[0].style.height='auto'; arguments[0].style.visibility='visible';";
    context.browser.execute_script(js, element)

#****************
# THEN STEPS
#****************

@then("wait for element {element} identified by {identifier}")
def step(context, element, identifier, timeout=10):
    find_element(context, element, identifier)

@then("check that the element {element} disappears, identified by {identifier}")
def step(context, element, identifier):
    wait_for_element_to_disappear(context, element, identifier)

@then("the page contains the text {expected}")
def step(context, expected):
    pass

@then("the page title must be {expected}")
def step(context, expected):
    assert context.browser.title == expected, "Error: Page title expected: {expected}, obtained: {obtained}".format(expected=expected, obtained=context.browser.title)

@then("the login error message appears saying {error_message}")
def step(context, expected):
    pass

@then("check that the combo button {combo_button} is selected")
def step(context, combo_button):
    if combo_button.is_selected():
        assert True
    else:
        assert False

@then("check that the element {selector} is not present")
def step(context, selector):
    try:
        element = find_element(context, selector, 'css_selector', timeout=1.0)
        assert False, "Error, the element is present, it is expected to be not present"
    except:
        assert True

@then("the text {text} is present in the element {element} identified by {identifier}")
def step(context, text, element, identifier):
    elem = find_element(context, element, identifier)
    assert elem.text == text, "Error: expected value is {exp}, but the actual value is {actual}".format(exp=text, actual=elem.text)

@then("the input field {input_field_element} contains the value {value} identified by {identifier}")
def step(context, input_field_element, value, identifier):
    elem = find_element(context, input_field_element, identifier)
    elem_value = elem.get_attribute("value")
    assert elem_value == value, "ERROR: Expected input {selector} to contain '{text}', but contains '{elem_text}'".format(selector=input_field_element, text=value, elem_text=elem_value)

@then("check that the element {element} is displayed, identified by {identifier}")
def step(context, element, identifier):
    elem = find_element(context, element, identifier)
    if elem.is_displayed():
        assert True
    if not elem.is_displayed():
        assert False, "The element is not displayed"

@then("wait until the element {element} is displayed, identified by {identifier}")
def step(context, element, identifier, timeout=5):
    elem = find_element(context, element, identifier)
    while timeout > 0:
        if elem.is_displayed():
            assert True
            return
        time.sleep(1)
        timeout -= 1
    assert False, "The element is not displayed"

@then("expect {selector} to contain {text}")
def step(context, selector, text):
    element = find_element(context, selector, 'css_selector')
    # lower(): when text is uppercased with CSS, it will return uppercase.
    assert text.lower() in element.text.lower(), "Expected {selector} to contain '{text}', but contains '{elem_text}'".format(text=text, selector=selector, elem_text=element.text)

@then("expect {selector} to have the value {text}")
def step(context, selector, text):
    element = find_element(context, selector, 'css_selector')
    assert text in element.get_attribute('value'), "Expected input {selector} to have the value '{text}', but has value '{elem_text}'".format(text=text, selector=selector, elem_text=element.get_attribute('value'))

@then("expect {selector} to disappear within {timeout} seconds")
def step(context, selector, timeout):
    wait_for_element_to_disappear(context, selector, 'css_selector', timeout=float(timeout))

@then("expect {selector} to disappear")
def step(context, selector):
    wait_for_element_to_disappear(context, selector, 'css_selector')

@then("allow time to update the UI")
def step(context):
    time.sleep(CHECK_INTERVAL_SECONDS)

@then("allow {time_seconds}s to update the UI")
def step(context, time_seconds):
    time.sleep(float(time_seconds))

@then("wait up to {timeout} seconds for {selector}")
def step(context, timeout, selector):
    wait_for_element_to_appear(context, selector, float(timeout))

@then("expect {selector} to have {option} selected")
def step(context, selector, option):
    element = find_element(context, selector, 'css_selector')
    select = Select(element)
    assert select.first_selected_option.text == option, \
        "Expected {option} to be selected, but {actual_option} was selected instead".format(
            option=option, actual_option=select.first_selected_option.text)

@then("wait for element {selector} to be clickable")
def step(context, selector):
    wait = WebDriverWait(context.browser, 10)
    wait.until(EC.element_to_be_clickable((By.CSS_SELECTOR, selector)))

@then("check that the combobox element {selector} selected value is {value}")
def step(context, selector, value):
    selected_option = Select(find_element(context, selector, 'css_selector')).first_selected_option.text
    if value == selected_option:
        assert True
    else:
        assert False, "Error, the option selected is '{observed}', expected '{obtained}'".format(observed=selected_option, obtained=value)

# These should be at the end of the file to prevent conflicts.
@then("expect {selector}")
def step(context, selector):
    element = find_element(context, selector, 'css_selector')
    assert element.is_displayed(), "Expected element {selector} isn't displayed".format(selector=selector)

@then("don't expect {selector}")
def step(context, selector):
    try:
        find_element(context, selector, 'css_selector', timeout=0).is_displayed()
    except (ElementNotVisibleException, NoSuchElementException):
        assert True
    else:
        assert False, "Element {selector} should not be displayed".format(selector=selector)

@then("wait for {selector}")
def step(context, selector, timeout=5):
    wait_for_element_to_appear(context, selector, 5.0)


import os
import time
import shutil
from selenium import webdriver
from selenium_basic_helpers import take_snapshot
from requests_oauthlib import OAuth1


def before_all(context):
    if context.config.userdata.get('test_type') == "selenium":
        if os.path.isfile("snapshots/snapshots_report.html"):
            shutil.rmtree('snapshots/')

def before_feature(context, feature):
    time.sleep(2)

    if context.config.userdata.get('test_type') == "selenium":
        desired_capabilities = {}
        browser_name = context.config.userdata.get('browser_name')
        if browser_name == "firefox":
            desired_capabilities = webdriver.DesiredCapabilities.FIREFOX
        elif browser_name == "chrome":
            desired_capabilities = webdriver.DesiredCapabilities.CHROME
        elif browser_name == "ie":
            desired_capabilities = webdriver.DesiredCapabilities.INTERNETEXPLORER
        elif browser_name == "opera":
            desired_capabilities = webdriver.DesiredCapabilities.OPERA
        elif browser_name == "phantomjs":
            desired_capabilities = webdriver.DesiredCapabilities.PHANTOMJS
        elif browser_name == "safari":
            desired_capabilities = webdriver.DesiredCapabilities.SAFARI
        elif browser_name == 'local':
            context.browser = webdriver.Firefox()
        if browser_name != 'local':
            desired_capabilities['name'] = str(feature.name + " - Environment: " + context.config.userdata.get('target_env'))
            desired_capabilities['os'] = context.config.userdata.get('operating_system')
            desired_capabilities['os_version'] = context.config.userdata.get('operating_system_version')
            desired_capabilities['browser_version'] = context.config.userdata.get('browser_version')
            desired_capabilities['resolution'] = context.config.userdata.get('screen_resolution')
            desired_capabilities['browserstack.autoWait'] = False
            desired_capabilities['browserstack.debug'] = True
            context.browser = webdriver.Remote(desired_capabilities=desired_capabilities, command_executor=context.config.userdata.get('webdriver_hub'))

    elif context.config.userdata.get('test_type') == "website_status":
        pass

    elif context.config.userdata.get('test_type') == "api":
        pass

    elif context.config.userdata.get('test_type') == "db":
        pass

    if context.config.userdata.get('test_type') == "selenium":
        context.browser.maximize_window()

def before_scenario(context, scenario):
    if context.config.userdata.has_key('wait_between_tests'):
        #print "Waiting " + context.config.wait_between_tests + " seconds before next scenario execution..."
        time.sleep(int(context.config.userdata.get('wait_between_tests')))
        #print "Running test now..."
    if context.config.userdata.get('test_type') == "db":
        pass

def after_scenario(context, scenario):
    pass

def after_step(context, step):
    if context.config.userdata.get('test_type') == "selenium":
        context.browser.title
        if step.status == "failed":
            take_snapshot(context, "base64", "STEP FAILED: " +step.name)

def after_feature(context, feature):
    if context.config.userdata.get('test_type') == "selenium":
        context.browser.quit()

def after_all(context):
    pass


import os
import time
import urlparse
from selenium.common.exceptions import *
from selenium.webdriver.common.by import By
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

#TODO: We should get this value from behave.ini
CHECK_INTERVAL_SECONDS = 0.01

def find_element(context, element, identifier, timeout=5.0):
    """
    Looks for a single web element in the page.
    :param context: the behave context variable.
    :param element: the element value to find.
    :param identifier: search by 'css_selector', 'id', 'name' or 'class_name'.
    :param timeout: optional parameter. Retries the search until n seconds (float).
    :return: selenium web element.
    """
    options = {
        'css_selector': context.browser.find_element_by_css_selector,
        'id': context.browser.find_element_by_id,
        'name': context.browser.find_element_by_name,
        'class_name': context.browser.find_element_by_class_name,
    }
    while timeout > 0:
        try:
            search_input = lambda: options[identifier](element)
            return search_input()
        except (ElementNotVisibleException, NoSuchElementException):
            time.sleep(CHECK_INTERVAL_SECONDS)
            timeout -= CHECK_INTERVAL_SECONDS
    search_input = lambda: options[identifier](element)
    return search_input()

def find_elements(context, selector, timeout=10):
    """
    Looks for a group of elements in the page.
    :param context: the behave context variable.
    :param selector: the css selector value to find.
    :param timeout: optional parameter. Retries the search until n seconds (float).
    :return: List of selenium web elements if found. If not elements found it will return an empty list.
    """
    while timeout > 0:
        elements = context.browser.find_elements_by_css_selector(selector)
        if elements != []:
            return elements
        else:
            time.sleep(1)
            timeout -= 1
    return []

def wait_for_element_to_appear(context, selector, timeout):
    """
    Wait the specified amount of time until the element appears.
    :param context: the behave context variable.
    :param selector: css_selector corresponding to the element.
    :param timeout: time to wait until the element appears.
    :return: Nothing, if the element does not appear in the specified time then an exception is raised.
    """
    element = find_element(context, selector, 'css_selector')
    while timeout > 0:
        if element.is_displayed():
            assert True
            return
        time.sleep(CHECK_INTERVAL_SECONDS)
        timeout -= CHECK_INTERVAL_SECONDS
    context.browser.refresh()
    raise Exception("The element {selector} is not displayed within {timeout} seconds".format(**locals()))

def wait_for_element_to_disappear(context, selector, identifier, timeout=10.0):
    """
    Wait n seconds until the specified element dissapears.
    :param context: the behave context variable.
    :param selector: the element value to find.
    :param identifier: search by 'css_selector', 'id', 'name' or 'class_name'.
    :param timeout: Optional. time to wait until the element dissapears.
    :return: True if the element dissapears. An exception is raised if the element does not dissapear.
    """
    while timeout >= 0:
        try:
            element = find_element(context, selector, identifier, timeout=0.0)
            if not element.is_displayed():
                return True
            time.sleep(CHECK_INTERVAL_SECONDS)
            timeout -= CHECK_INTERVAL_SECONDS
        except (ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException):
            return True
    assert False, "The element {selector} has not dissapeared within {timeout} seconds".format(**locals())

def scroll_element_into_view(driver, element):
    """
    Scrolls to an element.
    :param driver: the behave context.
    :param element: the web element object.
    :return: nothing.
    """
    y = element.location['y']
    driver.execute_script('window.scrollTo(0, {0})'.format(y))

def take_snapshot(context, output_format, text):
    """
    Takes an snapshot. It will be stored in a html file if the format is base64 or in a separate png file. The output directory will be snapshots.
    :param context: The behave context.
    :param output:format (base64 will add it to html file, or file to create a png file)
    :param text:label for base64, part of the file name in the case of file
    """
    if not os.path.exists("snapshots"):
        os.makedirs("snapshots")
    if output_format == "base64":
        captured_img = context.browser.get_screenshot_as_base64()
        if os.path.isfile("snapshots/snapshots_report.html"):
            with open("snapshots/snapshots_report.html", "a") as snapshot_file:
                snapshot_file.write("""<br>"""+ text +"""<br><img src="data:image/png;base64,"""+captured_img+"""" />""")
        else:
            with open("snapshots/snapshots_report.html", "w") as snapshot_file:
                snapshot_file.write("""<br>"""+ text +"""<br><img src="data:image/png;base64,"""+captured_img+"""" />""")
    if output_format == "file":
        captured_img = context.browser.get_screenshot_as_file("snapshots/" + str(time.time()) + text + ".png")
        if captured_img == False:
            raise Exception("ERROR: cannot take snapshot...")


from distutils.core import setup

setup(
    # Application name:
    name="behave_base_library",

    # Version number (initial):
    version="0.1.5",

    # Application author details:
    author="Juan Martin Moschino",
    author_email="jmoschino@gmail.com",

    # Packages
    packages=["behave_base_lib"],

    # Include additional files into the package
    include_package_data=True,

    # Details
    url="http://pypi.python.org/pypi/behave_base_lib_v010/",

    #
    # license="LICENSE.txt",
    description="Base files for any behave automation project",

    # long_description=open("README.txt").read(),

    # Dependent packages (distributions)
    #install_requires=[],
)



import requests

def erase_last_response(context):
    """
    Erase the content of the env variable for api responses (used by API methods)
    :param context: The working context
    :return: Nothing, it sets the variable an empty string value
    """
    context.config.userdata['last_response'] = ""

def set_api_response(context, response):
    """
    Set a value to the env variable for the last api response (used by API methods)
    :param context: The working context
    :param response: The last response
    :return: Nothing, it sets the response in the variable
    """
    context.config.userdata['last_response'] = response

def get_last_response(context):
    """
    Returns the last response stored in the variable (used by API methods)
    :param context: The working context
    :return: The last response
    """
    return context.config.userdata.get('last_response')

def get_method(context, url, check_response_code = True, check_message = None):
    """
    Calls the get method in the specified url
    :param context: The working context
    :param url: The url to get
    :param check_response_code: Verifies with an assert that the response is 200 http (default:True)
    :param check_message: Checks for the specified message in the response (default: None)
    :return: It stores the response in the variable in the api context, it can be obtained with get_last_response method
    """
    erase_last_response(context)
    response = requests.get(url)
    set_api_response(context, response)
    if check_response_code:
        assert response.status_code == 200, "Error, status code is {code}, expected 200. The error is {error}".format(code=response.status_code, error=response.text)
    if check_message != None:
        check_response_message(response, check_message)

def delete_method(context, url, check_response_code = True, check_message = None):
    """
    Calls the method delete in the specified url
    :param context: The working context
    :param url: The url to delete
    :param check_response_code: Verifies with an assert that the response is 200 http (default:True)
    :param check_message: Checks for the specified message in the response (default: None)
    :return: It stores the response in the variable api context, it can be obtained with get_last response method
    """
    erase_last_response(context)
    response = requests.delete(url)
    set_api_response(context, response)
    if check_response_code:
        assert response.status_code == 200, "Error, status code is {code}, expected 200. The error is {error}".format(code=response.status_code,  error=response.text)
    if check_message != None:
        check_response_message(response, check_message)

def post_method(context, url, data, check_response_code = True, check_message = None):
    """
    Calls the post method in the specified url with the specified payload
    :param context: The working context
    :param url: The url to post
    :param data: The payload
    :param check_response_code: Verifies with an assert that the response is 200 http (default:True)
    :param check_message: Checks for the specified message in the response (default: None)
    :return: It stores the response in the variable in the api context, it can be obtained with get_last_response method
    """
    erase_last_response(context)
    response = requests.post(url, data)
    set_api_response(context, response)
    if check_response_code:
        assert response.status_code == 200, "Error, status code is {code}, expected 200. The error is {error}".format(code=response.status_code, error=response.text)
    if check_message != None:
        check_response_message(response, check_message)

def put_method(context, url, data, check_response_code = True, check_message = None):
    """
    Calls the put method in the specified url with the specified payload
    :param context: The working context
    :param url: The url to put
    :param data: The payload
    :param check_response_code: Verifies with an assert that the response is 200 http (default:True)
    :param check_message: Checks for the specified message in the response (default: None)
    :return: It stores the response in the variable in the api context, it can be obtained with get_last_response method
    """
    erase_last_response(context)
    response = requests.put(url, data)
    set_api_response(context, response)
    if check_response_code:
        assert response.status_code == 200, "Error, status code is {code}, expected 200. The error is {error}".format(code=response.status_code, error=response.text)
    if check_message != None:
        check_response_message(response, check_message)

def check_response_message(api_object, message):
    """
    Verifies that the message is present in the api object
    :param api_object: The api object
    :param message: The message to check
    :return: Nothing, it throws an exception
    """
    if api_object.json()[message] != True:
        assert False, "ERROR, API error. Response {response}".format(response=api_object.json()['error'])